@@ -1,3 +1,20 @@
+"""
+CSV output
+
+Comma Separated Values output is easily processed with other command-line tools
+or imported into a spreadsheet or database.
+
+Python's `csv <https://docs.python.org/3/library/csv.html>`_ module is used to
+produce Excel-style CSV output, except with UNIX-style line endings. The field
+delimiter can be set with a formatting option, and date, number and base/quote
+formatting options will be respected.
+
+Classes:
+
+    CSV
+
+"""
+
 import csv
 import io
 
@@ -7,7 +24,7 @@ from .baseoutput import BaseOutput
 
 
 class CSV(BaseOutput):
-    def format(self, series, source=None, fmt=Format()):
+    def format(self, series, source, fmt=Format()):
         output = io.StringIO()
         writer = csv.writer(
             output,
